#include "os.h"

ULONG const FREQ  = 100; // 100 "ticks" per second
ULONG timer_ticks =   0;
ULONG eticks;

void timer_handler(struct regs* r)
{
    ++timer_ticks;
    if (eticks)
        --eticks;
}

void timer_wait (ULONG ticks)
{
    timer_uninstall();
    eticks = ticks;
    timer_install();

    // busy wait...
    while (eticks)
    {
        update_cursor();
    }
}

void sleepSeconds (ULONG seconds)
{
    timer_wait(FREQ * seconds);
}

void sleepMilliSeconds (ULONG ms)
{
    timer_wait(FREQ * ms/1000UL);
}

static void systemTimer_setFrequency( ULONG freq )
{
   ULONG divisor = 1193180 / freq; //divisor must fit into 16 bits

   // Send the command byte.
   outportb(0x43, 0x36);

   // Send divisor.
   outportb(0x40, (UCHAR)(  divisor     & 0xFF )); // low  byte
   outportb(0x40, (UCHAR)( (divisor>>8) & 0xFF )); // high byte
}

void timer_install()
{
    /* Installs 'timer_handler' to IRQ0 */
    irq_install_handler(0, timer_handler);
    systemTimer_setFrequency( FREQ ); // FREQ Hz, meaning a tick every 1000/FREQ milliseconds
}

void timer_uninstall()
{
    /* Uninstalls IRQ0 */
    irq_uninstall_handler(0);
}

